/************************************************************************/
/* 		This code is part of 'CutterFF'				*/
/*		a tool for cutting video files with 'FFmpeg' and 'GTK'	*/
/*		Copyright (c) 2017 .. 2019,2021 Harald Foerster		*/
/*		(harald_foerster@users.sourceforge.net)			*/
/*		Distributed under GPL 2 or later			*/
/************************************************************************/

#include <gdk/gdkkeysyms.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "cff_macros-gtk.h"
#include "cff_accel-gtk.h"
#include "cff_nls.h"
#include "cff_charconv.h"

/*
#include <gtk/gtk.h>
#include "cff_tools-gtk.h"
*/


enum
{
	kColumnMenu,
	kColumnKey,
	kColumnRowData,
	kNumberOfColumns
};


/* cff_gui.c */
extern void cff_gui_adjust_menu(GtkWidget *w, gpointer data);

static GtkAccelGroup *cff_gaccel_group = NULL;

static cff_gaccel_t cff_gaccel_tab[kGAccelKeyTotal] =
{
	{ "FileOpen",		NULL, NULL, 0, 0, 0 },
	{ "FileSave",		NULL, NULL, 0, 0, 0 },
	{ "FileImage",		NULL, NULL, 0, 0, 0 },
	{ "FileClose",		NULL, NULL, 0, 0, 0 },
	{ "FileInfo",		NULL, NULL, 0, 0, 0 },
	{ "FileSaverc",		NULL, NULL, 0, 0, 0 },
	{ "FileQuit",		NULL, NULL, 0, 0, 0 },
	{ "EditUndo",		NULL, NULL, 0, 0, 0 },
	{ "EditRedo",		NULL, NULL, 0, 0, 0 },
	{ "EditLoad",		NULL, NULL, 0, 0, 0 },
	{ "EditSave",		NULL, NULL, 0, 0, 0 },
	{ "EditClear",		NULL, NULL, 0, 0, 0 },
	{ "EditAdd",		NULL, NULL, 0, 0, 0 },
	{ "EditRemove",		NULL, NULL, 0, 0, 0 },
	{ "OptAccel",		NULL, NULL, 0, 0, 0 },
	{ "OptKeyframe",	NULL, NULL, 0, 0, 0 },
	{ "OptIframe",		NULL, NULL, 0, 0, 0 },
	{ "OptPframe",		NULL, NULL, 0, 0, 0 },
	{ "OptCursor",		NULL, NULL, 0, 0, 0 },
	{ "OptOverwrite",	NULL, NULL, 0, 0, 0 },
	{ "OptDelete",		NULL, NULL, 0, 0, 0 },
	{ "OptBeep",		NULL, NULL, 0, 0, 0 },
	{ "OptSave",		NULL, NULL, 0, 0, 0 },
	{ "HelpFFmpeg",		NULL, NULL, 0, 0, 0 },
	{ "HelpCutterff",	NULL, NULL, 0, 0, 0 },
	{ "HelpAbout",		NULL, NULL, 0, 0, 0 }

}; /* static cff_gaccel_t cff_gaccel_tab[kGAccelKeyTotal] */



typedef struct
{
	GtkWidget	*window;
	cff_gaccel_t	*rowdata;
	cff_gaccel_t	accel_tab[kGAccelKeyTotal];
	cff_ListStore_t	list;
	GtkTreeIter	iter;

} GaccelAssignDialog;

static GaccelAssignDialog *cff_gaccel_dialog = NULL;


static cff_gaccel_t *cff_gaccel_find_by_name(cff_gaccel_t *gaccelp, const gchar *name)
{
	gint loop = kGAccelKeyTotal - 1;

	do
	{
		gaccelp--;

		if (strcmp(gaccelp->name, name) == 0)
		{
			return gaccelp;
		}
	}
	while (--loop > kGAccelKeyNone);

	return NULL;

} /* static cff_gaccel_t *cff_gaccel_find_by_name(cff_gaccel_t*, const gchar*) */

static cff_gaccel_t *cff_gaccel_find_by_keys(cff_gaccel_t *gaccelp, guint keyval,
						GdkModifierType modifier)
{
	gint loop = kGAccelKeyTotal - 1;

	do
	{
		gaccelp--;

		if (keyval == gaccelp->keyval &&
				modifier == gaccelp->modifier)
		{
			return gaccelp;
		}
	}
	while (--loop > kGAccelKeyNone);

	return NULL;

} /* static cff_gaccel_t *cff_gaccel_find_by_keys(cff_gaccel_t*, guint, GdkModifierType) */

static gboolean cff_gaccel_key_pressed(GtkWidget *window,
					GdkEventKey *event, cff_gaccel_t *gaccelp)
{
	if (event->length && (event->state & GDK_CONTROL_MASK))
	{
		gaccelp = cff_gaccel_find_by_keys(gaccelp, event->keyval, event->state);

		if (gaccelp)
		{
			cff_gui_adjust_menu(gaccelp->widget, NULL);
		}
	}

	return FALSE;

} /* static gboolean cff_gaccel_key_pressed(GtkWidget*, GdkEventKey*, cff_gaccel_t*) */

static void cff_gaccel_remove_accelerator(const cff_gaccel_t *gaccelp,
						GtkAccelGroup *accel_group)
{
	gtk_accel_group_unlock(accel_group);

	gtk_widget_remove_accelerator(gaccelp->widget, accel_group,
						gaccelp->key, gaccelp->modifier);
	gtk_accel_group_lock(accel_group);

} /* static void cff_gaccel_remove_accelerator(const cff_gaccel_t*, GtkAccelGroup*) */

static void cff_gaccel_add_accelerator(const cff_gaccel_t *gaccelp,
						GtkAccelGroup *accel_group)
{
	gtk_accel_group_unlock(accel_group);

	gtk_widget_add_accelerator(gaccelp->widget, "activate", accel_group,
				gaccelp->key, gaccelp->modifier,  GTK_ACCEL_VISIBLE);

	gtk_accel_group_lock(accel_group);

} /* static void cff_gaccel_add_accelerator(const cff_gaccel_t*, GtkAccelGroup*) */

void cff_gaccel_bind_widget(gint accel_id, const gchar *label, GtkWidget *widget, GtkWidget *window)
{
	GtkAccelGroup	*accel_group;
	cff_gaccel_t	*gaccelp;

	if (accel_id == kGAccelKeyNone || widget == NULL)
	{
		return;
	}

	accel_group = cff_gaccel_group;

	if (accel_group == NULL)
	{
		accel_group  = gtk_accel_group_new();
		cff_gaccel_group = accel_group;
		gtk_accel_group_lock(accel_group);

		gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);

		GuiSignalConnect(window, "key-press-event",
			cff_gaccel_key_pressed, &cff_gaccel_tab[kGAccelKeyTotal]);
	}

	gaccelp = &cff_gaccel_tab[accel_id];

	gaccelp->label	= label;
	gaccelp->widget	= widget;

	if (gaccelp->key)
	{
		cff_gaccel_add_accelerator(gaccelp, accel_group);
	}

} /* void cff_gaccel_bind_widget(gint, const gchar*, GtkWidget*, GtkWidget*) */

static void cff_gaccel_dialog_destroy(GtkWidget *w, GaccelAssignDialog *acceldlg)
{
	g_free(acceldlg);
	cff_gaccel_dialog = NULL;

} /* static void cff_gaccel_dialog_destroy(GtkWidget*, GaccelAssignDialog*) */

static void cff_gaccel_dialog_okay(GtkWidget *w, GaccelAssignDialog *acceldlg)
{
	gint	 	loop;
	cff_gaccel_t	*old, *mod;

	old	= cff_gaccel_tab;
	mod 	= acceldlg->accel_tab;
	loop	= 0;
	
	do
	{
		if (old->keyval != mod->keyval || old->modifier != mod->modifier)
		{
			if (old->keyval)
			{
				cff_gaccel_remove_accelerator(old, cff_gaccel_group);
			}
		}

		old++;
		mod++;
	}
	while (++loop < kGAccelKeyTotal);

	old	= cff_gaccel_tab;
	mod 	= acceldlg->accel_tab;
	loop	= 0;
	
	do
	{
		if (old->keyval != mod->keyval || old->modifier != mod->modifier)
		{
			if (mod->keyval)
			{
				cff_gaccel_add_accelerator(mod, cff_gaccel_group);
			}

			old->keyval   = mod->keyval;
			old->key      = mod->key;
			old->modifier = mod->modifier;
		}

		old++;
		mod++;
	}
	while (++loop < kGAccelKeyTotal);

	gtk_widget_destroy(acceldlg->window);

} /* static void cff_gaccel_dialog_okay(GtkWidget*, GaccelAssignDialog*) */

static gboolean cff_gaccel_dialog_key(GtkWidget *window,
			GdkEventKey *event, GaccelAssignDialog *acceldlg)
{
	cff_gaccel_t *gaccelp;

	if (event->keyval == Gui_KEY_BackSpace)
	{
		gaccelp = acceldlg->rowdata;

		if (gaccelp->keyval)
		{
			gtk_list_store_set(acceldlg->list.store,
				&(acceldlg->iter), kColumnKey, NULL, -1);

			gaccelp->keyval   = 0;
			gaccelp->key      = 0;
			gaccelp->modifier = 0;
		}

		return TRUE;
	}

	else if (event->length && (event->state & GDK_CONTROL_MASK))
	{
		if (cff_gaccel_find_by_keys(&acceldlg->accel_tab[kGAccelKeyTotal],
							event->keyval, event->state))
		{
			cff_guitl_beep();
		}

		else
		{
			gint key = event->keyval;

			if (event->state & GDK_SHIFT_MASK)
			{
				key = gdk_keyval_to_lower(key);
			}

			if (gtk_accelerator_valid(key, event->state))
			{
				gchar *name = gtk_accelerator_name(key, event->state);

				if (name)
				{
					gchar *str;

					str = cff_charconv_utf8_from_locale(name, NULL);

					gtk_list_store_set(acceldlg->list.store,
						&(acceldlg->iter), kColumnKey, str, -1);
					CFF_CHARCONV_UTF8_FREE(str, name);
					g_free(name);

					gaccelp = acceldlg->rowdata;

					gaccelp->keyval   = event->keyval;
					gaccelp->key      = key;
					gaccelp->modifier = event->state;
				}
			}

		} /* if (cff_gaccel_find_by_keys(..) .. else */

		return TRUE;

	} /* if (event->keyval == Gui_KEY_BackSpace) .. else if (..) */

	return FALSE;

} /* static gboolean cff_gaccel_dialog_key(GtkWidget*, GdkEventKey*, GaccelAssignDialog*) */

static void cff_gaccel_dialog_row_selected(GtkTreeSelection *selection, GaccelAssignDialog *acceldlg)
{
	GtkTreeIter	iter;
	GtkTreeModel	*model;

	if (gtk_tree_selection_get_selected(selection, &model, &iter))
	{
		acceldlg->iter = iter;
		gtk_tree_model_get(model, &iter, kColumnRowData, &(acceldlg->rowdata), -1);
	}

} /* static void cff_gaccel_dialog_row_selected(GtkTreeSelection*, GaccelAssignDialog*) */

static void cff_gaccel_build_column(cff_ListStore_t *list, gint colnr, const gchar *title)
{
	gchar			*str;
	GtkTreeViewColumn	*column;
	GtkCellRenderer		*renderer;

	column = gtk_tree_view_column_new();

	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(column, renderer, FALSE);
	gtk_tree_view_column_set_attributes(column, renderer, "text", colnr, NULL);

	gtk_tree_view_append_column(list->view, column);

	str = cff_charconv_utf8_gettext(title);
	gtk_tree_view_column_set_title(column, str);
	CFF_CHARCONV_UTF8_FREE(str, title);

	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);

} /* static void cff_gaccel_build_column(cff_ListStore_t*, gint, const gchar*) */

void cff_gaccel_dialog_open(void)
{
	gint		loop;
	GtkWidget	*button, *frame, *hbox, *label, *swin, *vbox, *window;

	GaccelAssignDialog *acceldlg = cff_gaccel_dialog;

	if (acceldlg)
	{
		gtk_window_present(GTK_WINDOW(acceldlg->window));
		return;
	}

	acceldlg = g_malloc(sizeof(GaccelAssignDialog));
	cff_gaccel_dialog = acceldlg;

	vbox = GuiVBoxNew(FALSE, 2);
	window = cff_guitl_window_new_gettext(gettext_nop("CutterFF: Keyboard"), vbox, 4);
	acceldlg->window = window;

	GuiSignalConnect(window, "destroy", cff_gaccel_dialog_destroy, acceldlg);
	GuiSignalConnect(window, "key-press-event", cff_gaccel_dialog_key, acceldlg);

	acceldlg->list.store =
		gtk_list_store_new(kNumberOfColumns, G_TYPE_STRING,
					G_TYPE_STRING, G_TYPE_POINTER);

	acceldlg->list.view = (GtkTreeView *)
		gtk_tree_view_new_with_model(GTK_TREE_MODEL(acceldlg->list.store));

	acceldlg->list.selection = gtk_tree_view_get_selection(acceldlg->list.view);
	g_object_unref(G_OBJECT(acceldlg->list.store));

	swin = cff_guitl_scrolled_window_new(vbox, 320, 180, GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER(swin), GTK_WIDGET(acceldlg->list.view));

	cff_gaccel_build_column(&(acceldlg->list), kColumnMenu, gettext_nop("Menu"));
	cff_gaccel_build_column(&(acceldlg->list), kColumnKey, gettext_nop("Key"));

	memcpy(acceldlg->accel_tab, cff_gaccel_tab, sizeof(cff_gaccel_tab));

	loop = 0;

	do
	{
		gchar		*str[2], *title;
		GtkTreeIter	iter;

		str[0] = cff_charconv_utf8_gettext(acceldlg->accel_tab[loop].label);
		title = gtk_accelerator_name(acceldlg->accel_tab[loop].key,
						acceldlg->accel_tab[loop].modifier);
		str[1] = cff_charconv_utf8_from_locale(title, NULL);

		gtk_list_store_append(acceldlg->list.store, &iter);
		gtk_list_store_set(acceldlg->list.store, &iter,
			kColumnMenu, str[0],
			kColumnKey, str[1],
			kColumnRowData, &(acceldlg->accel_tab[loop]),
			-1);

		CFF_CHARCONV_UTF8_FREE(str[0], acceldlg->accel_tab[loop].label);
		CFF_CHARCONV_UTF8_FREE(str[1], title);
		g_free(title);

		if (loop == 0)
		{
			gtk_tree_selection_select_iter(acceldlg->list.selection, &iter);
		}
	}
	while (++loop < kGAccelKeyTotal);

	g_object_set(acceldlg->list.view,
			"headers-visible", TRUE,
			"headers-clickable", FALSE,
			NULL);

	gtk_tree_selection_set_mode(acceldlg->list.selection,
						GTK_SELECTION_BROWSE);

	GuiSignalConnect(acceldlg->list.selection, "changed",
				cff_gaccel_dialog_row_selected, acceldlg);

	label = cff_guitl_label_new_gettext(
			gettext_nop("Select a menu entry and press your prefered keys,"
							" e.g. Quit => 'Cntrl' + 'Q'"));
	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);

	frame = cff_guitl_frame_new_add_child_gettext(NULL, label);
	GuiVBoxPackStart(vbox, frame, FALSE, FALSE, 1);

	hbox = cff_guitl_hbutton_box_new(vbox, 8, GTK_BUTTONBOX_END);

	button = cff_guitl_button_box_add_button(hbox,
				gettext_nop("Cancel"), NULL, NULL);
	GuiSignalConnectSwapped(button, "clicked",
			gtk_widget_destroy, acceldlg->window);

	button = cff_guitl_button_box_add_button(hbox, gettext_nop("OK"),
			 	(GCallback) cff_gaccel_dialog_okay, acceldlg);
	gtk_widget_grab_default(button);

	gtk_widget_show_all(acceldlg->window);

} /* void cff_gaccel_dialog_open(void) */

gint cff_gaccel_parse_str(gchar *str)
{
	gchar *ptr = strchr(str, '=');

	if (ptr != NULL)
	{
		gchar *peq, *name;

		peq = ptr,
		name = str;
		*(ptr++) = '\0';

		if (*(ptr++) == '"')
		{
			gchar *accelerator = ptr;

			ptr = strrchr(accelerator, '"');

			if (ptr != NULL)
			{
				guint		accel_key;
				GdkModifierType	modifier;

				*ptr      = '\0';
				modifier  = 0;
				accel_key = 0;

				gtk_accelerator_parse(accelerator, &accel_key, &modifier);

				if (accel_key != 0 && (modifier & GDK_CONTROL_MASK))
				{
					cff_gaccel_t *gaccelp;

					gaccelp = cff_gaccel_find_by_name(&cff_gaccel_tab[kGAccelKeyTotal], name);

					if (gaccelp != NULL)
					{
						gaccelp->modifier = modifier;
						gaccelp->key = accel_key;

						gaccelp->keyval = (modifier & GDK_SHIFT_MASK) ?
						gdk_keyval_to_upper(accel_key) : accel_key;

						return TRUE;
					}

				} /* if (accel_key != 0 && (modifier & GDK_CONTROL_MASK)) */

				*ptr = '"';

			} /* if (ptr != NULL) */

		} /* if (*(ptr++) != '"') */

		*peq = '=';

	} /* if (ptr != NULL) */

	return FALSE;

} /* gint cff_gaccel_parse_str(gchar*) */

void cff_gaccel_save_data(FILE *fd, const gchar *id)
{
	cff_gaccel_t *gaccelp = cff_gaccel_tab;

	gint loop = kGAccelKeyTotal - 1;

	do
	{
		if (gaccelp->key)
		{
			gchar *ptr = gtk_accelerator_name(gaccelp->key, gaccelp->modifier);

			if (ptr != NULL)
			{
				fprintf(fd, "%s %s=\"%s\"\n", id, gaccelp->name, ptr);
				g_free(ptr);
			}
		}

		gaccelp++;
	}
	while (--loop > kGAccelKeyNone);

} /* void cff_gaccel_save_data(FILE*, const gchar*) */

/* EOF */
