/************************************************************************/
/* 		This code is part of 'CutterFF'				*/
/*		a tool for cutting video files with 'FFmpeg' and 'GTK'	*/
/*		Copyright (c) 2017,2018,2021 Harald Foerster		*/
/*		(harald_foerster@users.sourceforge.net)			*/
/*		Distributed under GPL 2 or later			*/
/************************************************************************/

#include <string.h>

#include "cff_nls.h"
#include "cff_charconv.h"
#include "cff_tools-gtk.h"

/*
#include <glib.h>
*/

gchar *cff_charconv_utf8_from_locale(const gchar *str, gint *plen)
{
	if (str != NULL && str[0] != '\0' &&
				g_utf8_validate(str, -1, NULL) == FALSE)
	{
		gssize	slen;
		gchar	*utf;
		gsize	written;

		if (plen == NULL)
		{
			slen = -1;
		}

		else
		{
			slen = *plen;
		}

		utf = g_locale_to_utf8(str, slen, NULL, &written, NULL);

		if (utf == NULL)
		{
			utf = g_strdup(str);
			utf = cff_tl_replace_non_ascii_chars_in_str(utf);
		}

		else if (plen != NULL)
		{
			*plen = written;
		}

		return utf;

	} /*  if (str != NULL && str[0] != '\0' &&
				g_utf8_validate(str, -1, NULL) == FALSE) */

	return (gchar *) str;

} /* gchar *cff_charconv_utf8_from_locale(const gchar*, gint*) */

gchar *cff_charconv_utf8_gettext(const gchar *str)
{
	if (str != NULL)
	{
		gchar *utf, *nls;

		nls = gettext(str);
		utf = cff_charconv_utf8_from_locale(nls, NULL);

		if (utf != str)
		{
			if (utf == nls)
			{
				return g_strdup(utf);
			}

			if (utf != NULL)
			{
				return utf;
			}
		}

	} /* if (str != NULL) */

	return (gchar *) str;

} /* gchar *cff_charconv_utf8_gettext(const gchar*) */

gchar *cff_charconv_utf8_from_filename(const gchar *str)
{
	if (str != NULL && str[0] != '\0' &&
			g_utf8_validate(str, -1, NULL) == FALSE)
	{

#ifdef BUILD_MINGW
		/* Windows GLib and ffmpeg are using UTF-8 for filenames */
		return g_locale_to_utf8(str, -1, NULL, NULL, NULL);
#else
		/* Other platforms indirectly depends on the current locale */
		return g_filename_to_utf8(str, -1, NULL, NULL, NULL);
#endif
	}

	return (gchar *) str;

} /* gchar *cff_charconv_utf8_from_filename(const gchar *str) */

gchar *cff_charconv_utf8_to_filename(const gchar *str)
{
	if (str != NULL && str[0] != '\0')
	{
		gboolean retv = g_utf8_validate(str, -1, NULL);

#ifdef BUILD_MINGW
		/* Windows GLib and ffmpeg are using UTF-8 for filenames */
		if (retv == FALSE)
		{
			return g_locale_to_utf8(str, -1, NULL, NULL, NULL);
		}
#else
		/* Other platforms indirectly depends on the current locale */
		if (retv == TRUE)
		{
			return g_filename_from_utf8(str, -1, NULL, NULL, NULL);
		}
#endif
	}

	return (gchar *) str;

} /* gchar *cff_charconv_utf8_to_filename(const gchar*) */

/* EOF */
