/************************************************************************/
/* 		This code is part of 'CutterFF'				*/
/*		a tool for cutting video files with 'FFmpeg' and 'GTK'	*/
/*		Copyright (c) 2017 .. 2019,2021 Harald Foerster		*/
/*		(harald_foerster@users.sourceforge.net)			*/
/*		Distributed under GPL 2 or later			*/
/************************************************************************/

#ifndef CFF_MACROS_GTK__H
#define CFF_MACROS_GTK__H

#include "cff_tools-gtk.h"

/*
#include <gtk/gtk.h>
*/

void cff_gui_init(const char *fname, const char *loglevel);


/*
	--------
	 Macros
	--------
*/

#if GDK_MAJOR_VERSION == 2

#define Gui_KEY_BackSpace	GDK_BackSpace
#define Gui_KEY_Escape		GDK_Escape
#define Gui_KEY_KP_1		GDK_KP_1
#define Gui_KEY_KP_2		GDK_KP_2
#define Gui_KEY_KP_3		GDK_KP_3
#define Gui_KEY_KP_4		GDK_KP_4
#define Gui_KEY_KP_6		GDK_KP_6
#define Gui_KEY_KP_7		GDK_KP_7
#define Gui_KEY_KP_8		GDK_KP_8
#define Gui_KEY_KP_9		GDK_KP_9
#define Gui_KEY_Home		GDK_Home
#define Gui_KEY_End		GDK_End
#define Gui_KEY_Page_Up		GDK_Page_Up
#define Gui_KEY_Page_Down	GDK_Page_Down
#define Gui_KEY_Up		GDK_Up
#define Gui_KEY_Down		GDK_Down
#define Gui_KEY_Left		GDK_Left
#define Gui_KEY_Right		GDK_Right
#define Gui_KEY_KP_Home		GDK_KP_Home
#define Gui_KEY_KP_End		GDK_KP_End
#define Gui_KEY_KP_Page_Up	GDK_KP_Page_Up
#define Gui_KEY_KP_Page_Down	GDK_KP_Page_Down
#define Gui_KEY_KP_Up		GDK_KP_Up
#define Gui_KEY_KP_Down		GDK_KP_Down
#define Gui_KEY_KP_Left		GDK_KP_Left
#define Gui_KEY_KP_Right	GDK_KP_Right

#else /* GDK_MAJOR_VERSION != 2 */

#define Gui_KEY_BackSpace	GDK_KEY_BackSpace
#define Gui_KEY_Escape		GDK_KEY_Escape
#define Gui_KEY_KP_1		GDK_KEY_KP_1
#define Gui_KEY_KP_2		GDK_KEY_KP_2
#define Gui_KEY_KP_3		GDK_KEY_KP_3
#define Gui_KEY_KP_4		GDK_KEY_KP_4
#define Gui_KEY_KP_6		GDK_KEY_KP_6
#define Gui_KEY_KP_7		GDK_KEY_KP_7
#define Gui_KEY_KP_8		GDK_KEY_KP_8
#define Gui_KEY_KP_9		GDK_KEY_KP_9
#define Gui_KEY_Home		GDK_KEY_Home
#define Gui_KEY_End		GDK_KEY_End
#define Gui_KEY_Page_Up		GDK_KEY_Page_Up
#define Gui_KEY_Page_Down	GDK_KEY_Page_Down
#define Gui_KEY_Up		GDK_KEY_Up
#define Gui_KEY_Down		GDK_KEY_Down
#define Gui_KEY_Left		GDK_KEY_Left
#define Gui_KEY_Right		GDK_KEY_Right
#define Gui_KEY_KP_Home		GDK_KEY_KP_Home
#define Gui_KEY_KP_End		GDK_KEY_KP_End
#define Gui_KEY_KP_Page_Up	GDK_KEY_KP_Page_Up
#define Gui_KEY_KP_Page_Down	GDK_KEY_KP_Page_Down
#define Gui_KEY_KP_Up		GDK_KEY_KP_Up
#define Gui_KEY_KP_Down		GDK_KEY_KP_Down
#define Gui_KEY_KP_Left		GDK_KEY_KP_Left
#define Gui_KEY_KP_Right	GDK_KEY_KP_Right

#endif /* #if GDK_MAJOR_VERSION == 2 .. #else */


#if GTK_MAJOR_VERSION == 2
#define Gui_STATE_NORMAL	GTK_STATE_NORMAL
#define Gui_STATE_PRELIGHT	GTK_STATE_PRELIGHT
#else
#define Gui_STATE_NORMAL	GTK_STATE_FLAG_NORMAL
#define Gui_STATE_PRELIGHT	GTK_STATE_FLAG_PRELIGHT
#endif

#if GTK_MAJOR_VERSION == 2
#define Gui_ALIGN_START		((gfloat) 0.0)
#define Gui_ALIGN_CENTER	((gfloat) 0.5)
#else
#define Gui_ALIGN_START		GTK_ALIGN_START
#define Gui_ALIGN_CENTER	GTK_ALIGN_CENTER
#endif

#if GTK_MAJOR_VERSION >= 3
#define Gui_MARGIN_TOP		"margin-top"
#define Gui_MARGIN_BOTTOM	"margin-bottom"
#define Gui_MARGIN_LEFT		"margin-left"
#define Gui_MARGIN_RIGHT	"margin-right"
#define Gui_MARGIN_START	"margin-start"
#define Gui_MARGIN_END		"margin-end"
#endif

#ifdef GDK_PRIORITY_REDRAW
#define Gui_PRIORITY_REDRAW	GDK_PRIORITY_REDRAW
#else
#define Gui_PRIORITY_REDRAW	(G_PRIORITY_HIGH_IDLE + 20)
#endif


/*
	*
	* ---------------
	*  Signal Macros
	* ---------------
	*
*/

#define GuiSignalFunc	GCallback

#define GuiSignalConnect(instance, name, func, data) \
	g_signal_connect((gpointer) (instance), (name), G_CALLBACK(func), (data))
#define GuiSignalConnectSwapped(instance, name, func, swap_obj) \
	g_signal_connect_swapped((gpointer) (instance), (name), \
					G_CALLBACK(func), G_OBJECT(swap_obj))
#define GuiSignalEmitStopByName(instance, name) \
	g_signal_stop_emission_by_name((gpointer) (instance), (name))
#define GuiSignalHandlerBlock(instance, id) \
	g_signal_handler_block((gpointer) (instance), (id))
#define GuiSignalHandlerUnblock(instance, id) \
	g_signal_handler_unblock((gpointer) (instance), (id))

#ifdef G_SOURCE_CONTINUE
#define Gui_SOURCE_CONTINUE	G_SOURCE_CONTINUE
#else
#define Gui_SOURCE_CONTINUE	TRUE
#endif

#ifdef G_SOURCE_REMOVE
#define Gui_SOURCE_REMOVE	G_SOURCE_REMOVE
#else
#define Gui_SOURCE_REMOVE	FALSE
#endif


/*
	*
	* ---------
	*  Threads
	* ---------
	*
*/

#if GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION < 32

#define GuiThreadNew(func, data) \
	g_thread_create((GThreadFunc) (func), (data), TRUE, NULL)

#define GuiMutex	 GStaticMutex

#define GuiMutexInit(mutex) \
	g_static_mutex_init(mutex)

#define GuiMutexFree(mutex) \
	g_static_mutex_free(mutex)

#define GuiMutexLock(mutex) \
	g_static_mutex_lock(mutex)

#define GuiMutexTryLock(mutex) \
	g_static_mutex_trylock(mutex)

#define GuiMutexUnlock(mutex) \
	g_static_mutex_unlock(mutex)

#else /* GLIB_MAJOR_VERSION != 2 || GLIB_MINOR_VERSION >= 32 */

#define GuiThreadNew(func, data) \
	g_thread_try_new(NULL, (GThreadFunc) (func), (data), NULL)

#define GuiMutex	 GMutex

#define GuiMutexInit(mutex) \
	g_mutex_init(mutex)

#define GuiMutexFree(mutex) \
	g_mutex_clear(mutex)

#define GuiMutexLock(mutex) \
	g_mutex_lock(mutex)

#define GuiMutexTryLock(mutex) \
	g_mutex_trylock(mutex)

#define GuiMutexUnlock(mutex) \
	g_mutex_unlock(mutex)

#endif /* #if GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION < 32 .. #else */


/*
	*
	* ---------------
	*  Miscellaneous
	* ---------------
	*
*/

#if GTK_MAJOR_VERSION == 2
#define GuiAdjustmentGetUpper(adj) \
	(adj)->upper
#else
#define GuiAdjustmentGetUpper(adj) \
	gtk_adjustment_get_upper(adj)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiAdjustmentGetValue(adj) \
	(adj)->value
#else
#define GuiAdjustmentGetValue(adj) \
	gtk_adjustment_get_value(adj)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiAdjustmentGetPageSize(adj) \
	(adj)->page_size
#else
#define GuiAdjustmentGetPageSize(adj) \
	gtk_adjustment_get_page_size(adj)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiColorParse(spec, color) \
	gdk_color_parse((spec), (color))
#else
#define GuiColorParse(spec, color) \
	gdk_rgba_parse((color), (spec))
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiColorToString(color) \
	gdk_color_to_string(color)
#else
#define GuiColorToString(color) \
	gdk_rgba_to_string(color)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiColorButtonNewWithColor(color) \
	gtk_color_button_new_with_color(color)
#else
#define GuiColorButtonNewWithColor(color) \
	gtk_color_button_new_with_rgba(color)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiCursorUnref(cursor) \
	gdk_cursor_unref(cursor)
#else
#define GuiCursorUnref(cursor) \
	g_object_unref(cursor)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiDialogSetModal(dialog, parent) \
{ \
	gtk_window_set_modal(GTK_WINDOW(dialog), TRUE); \
	gtk_window_set_transient_for(GTK_WINDOW(dialog), \
					GTK_WINDOW(parent)); \
}
#else
#define GuiDialogSetModal(dialog, parent) \
	g_object_set(G_OBJECT(dialog), "modal", TRUE, \
			"transient-for", (parent), NULL)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiImageGetDialogIcon(name) \
	gtk_image_new_from_stock((name), GTK_ICON_SIZE_DIALOG)
#else
#define GuiImageGetDialogIcon(name) \
	gtk_image_new_from_icon_name((name), GTK_ICON_SIZE_DIALOG)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiMiscSetAlignment(widget, xalign, yalign) \
	gtk_misc_set_alignment(GTK_MISC(widget), (xalign), (yalign))
#else
#define GuiMiscSetAlignment(widget, xalign, yalign) \
	g_object_set(G_OBJECT(widget), "halign", (xalign), \
			"valign", (yalign), NULL)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiMiscSetPadding(widget, padding) \
	gtk_misc_set_padding(GTK_MISC(widget), (padding), (padding))
#else
#define GuiMiscSetPadding(widget, padding) \
	g_object_set(G_OBJECT(widget), "margin", (padding), NULL)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiProgressbarTextHide(pbar) \
	gtk_progress_bar_set_text((pbar), NULL)
#else
#define GuiProgressbarTextHide(pbar) \
	gtk_progress_bar_set_show_text((pbar), 0)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiHScaleNew(adjust) \
	gtk_hscale_new(adjust)
#else
#define GuiHScaleNew(adjust) \
	gtk_scale_new(GTK_ORIENTATION_HORIZONTAL, (adjust))
#endif

/* Combo */

#if GTK_MAJOR_VERSION == 2
#define GuiOptionMenu	GtkOptionMenu
#else
#define GuiOptionMenu	GtkComboBox
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiOptionMenuTextNew() \
	gtk_option_menu_new()
#else
#define GuiOptionMenuTextNew() \
	gtk_combo_box_text_new()
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiOptionMenuTextAppend(option, menu, index, str) \
{ \
	GtkWidget *mi = gtk_menu_item_new_with_label(str); \
	gtk_menu_append(GTK_MENU(menu), mi); \
	gtk_widget_show(mi); \
}
#else
#define GuiOptionMenuTextAppend(option, menu, index, str) \
	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(option), str)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiOptionMenuGetActive(option) \
	gtk_option_menu_get_history(GTK_OPTION_MENU(option))
#else
#define GuiOptionMenuGetActive(option) \
	gtk_combo_box_get_active(GTK_COMBO_BOX(option))
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiOptionMenuSetActive(option, index) \
	gtk_option_menu_set_history(GTK_OPTION_MENU(option), (index))
#else
#define GuiOptionMenuSetActive(option, index) \
	gtk_combo_box_set_active(GTK_COMBO_BOX(option), (index))
#endif

/* Menu */

#if GTK_MAJOR_VERSION == 2
#define GuiCheckMenuItemGetActive(item) \
	(item)->active
#else
#define GuiCheckMenuItemGetActive(item) \
	gtk_check_menu_item_get_active(item)
#endif

#define GuiMenuBarDisableShadow(menu) \
{ \
	gtk_rc_parse_string( \
		"style \"cutterff-menubar-style\" {GtkMenuBar::shadow-type = none}" \
		"class \"GtkMenuBar\" style : highest \"cutterff-menubar-style\" "); \
	gtk_widget_set_name((menu), "cutterff-menubar-style"); \
}

#if GTK_MAJOR_VERSION == 2
#define GuiMenuSeparatorNew() \
	gtk_menu_item_new()
#else
#define GuiMenuSeparatorNew() \
	gtk_separator_menu_item_new()
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiMenuShellGetChildren(menu_shell) \
	GTK_MENU_SHELL(menu_shell)->children
#else
#define GuiMenuShellGetChildren(menu_shell) \
	gtk_container_get_children(GTK_CONTAINER(menu_shell))
#endif

/* Widgets */

#if GTK_MAJOR_VERSION == 2
#define GuiWidgetGetAllocation(widget, alloc) \
	*(alloc) = (widget)->allocation
#else
#define GuiWidgetGetAllocation(widget, alloc) \
	gtk_widget_get_allocation((widget), (alloc))
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiWidgetGetAllocatedHeight(widget) \
	(widget)->allocation.height
#else
#define GuiWidgetGetAllocatedHeight(widget) \
	gtk_widget_get_allocated_height(widget)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiWidgetGetAllocatedWidth(widget) \
	(widget)->allocation.width
#else
#define GuiWidgetGetAllocatedWidth(widget) \
	gtk_widget_get_allocated_width(widget)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiWidgetGetWindow(widget) \
	(widget)->window
#else
#define GuiWidgetGetWindow(widget) \
	gtk_widget_get_window(widget)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiWidgetSetState(widget, flags) \
	gtk_widget_set_state((widget), (flags))
#else
#define GuiWidgetSetState(widget, flags) \
	gtk_widget_set_state_flags((widget), (flags), TRUE)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiWidgetCanDefault(widget) \
	GTK_WIDGET_SET_FLAGS((widget), GTK_CAN_DEFAULT)
#else
#define GuiWidgetCanDefault(widget) \
	gtk_widget_set_can_default((widget), TRUE)
#endif

/* Boxes */

#if GTK_MAJOR_VERSION == 2
#define GuiHButtonBoxNew() \
	gtk_hbutton_box_new()
#else
#define GuiHButtonBoxNew() \
	gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiButtonBoxSetSpacing(widget, spacing) \
	gtk_box_set_spacing(GTK_BOX(widget), (spacing))
#else
#define GuiButtonBoxSetSpacing(widget, spacing) \
	g_object_set(G_OBJECT(widget), "spacing", (spacing), NULL)
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiHBoxNew(homogeneous, spacing) \
	gtk_hbox_new((homogeneous), (spacing))
#else
#define GuiHBoxNew(homogeneous, spacing) \
	cff_guitl_hgrid_new((homogeneous), (spacing))
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiHBoxPackStart(box, child, expand, fill, padding) \
	gtk_box_pack_start(GTK_BOX(box), (child), (expand), (fill), (padding))
#elif GTK_MAJOR_VERSION == 3
#define GuiHBoxPackStart(box, child, expand, fill, padding) \
	{ \
		const gchar *start, *end; \
		if (gtk_get_minor_version() < 12) \
		{ \
			start = Gui_MARGIN_LEFT; \
			end   = Gui_MARGIN_RIGHT; \
		} \
		else \
		{ \
			start = Gui_MARGIN_START; \
			end   = Gui_MARGIN_END; \
		} \
		g_object_set(G_OBJECT(child), "hexpand", (expand), \
			"halign", (((fill) == FALSE) ? \
				GTK_ALIGN_CENTER : GTK_ALIGN_FILL), \
					start, (padding), \
						end, (padding), NULL); \
		gtk_container_add(GTK_CONTAINER(box), (child)); \
	}
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiVBoxNew(homogeneous, spacing) \
	gtk_vbox_new((homogeneous), (spacing))
#else
#define GuiVBoxNew(homogeneous, spacing) \
	cff_guitl_vgrid_new((homogeneous), (spacing))
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiVBoxPackStart(box, child, expand, fill, padding) \
	gtk_box_pack_start(GTK_BOX(box), (child), (expand), (fill), (padding))
#else
#define GuiVBoxPackStart(box, child, expand, fill, padding) \
	g_object_set(G_OBJECT(child), "vexpand", (expand), \
		"valign", (((fill) == FALSE) ? \
			GTK_ALIGN_CENTER : GTK_ALIGN_FILL), \
				Gui_MARGIN_TOP, (padding), \
					Gui_MARGIN_BOTTOM, (padding), NULL), \
	gtk_container_add(GTK_CONTAINER(box), (child))
#endif

#if GTK_MAJOR_VERSION == 2
#define GuiHBoxSetSpacing(box, spacing) \
	gtk_box_set_spacing(GTK_BOX(box), (spacing))
#elif GTK_MAJOR_VERSION == 3
#define GuiHBoxSetSpacing(box, spacing) \
	{ \
		const gchar *start, *end; \
		if (gtk_get_minor_version() < 12) \
		{ \
			start = Gui_MARGIN_LEFT; \
			end   = Gui_MARGIN_RIGHT; \
		} \
		else \
		{ \
			start = Gui_MARGIN_START; \
			end   = Gui_MARGIN_END; \
		} \
		g_object_set(G_OBJECT(box), start, (spacing), \
					end, (spacing), NULL); \
	}
#endif

#endif /* CFF_MACROS_GTK__H */
